@@ -28,11 +28,11 @@ from urllib import parse as urlparse
 
 from oslo_concurrency import processutils
 from oslo_config import cfg
-from oslo_service import loopingcall
 from oslo_utils import excutils
 from oslo_utils import specs_matcher
 from oslo_utils import strutils
 from oslo_utils import units
+import tenacity
 
 from ironic_lib.common.i18n import _
 from ironic_lib import exception
@@ -96,13 +96,42 @@ def execute(*cmd, **kwargs):
         else:
             kwargs['root_helper'] = CONF.ironic_lib.root_helper
 
-    result = processutils.execute(*cmd, **kwargs)
-    LOG.debug('Execution completed, command line is "%s"',
-              ' '.join(map(str, cmd)))
-    if log_stdout:
-        LOG.debug('Command stdout is: "%s"', result[0])
-    LOG.debug('Command stderr is: "%s"', result[1])
-    return result
+    def _log(stdout, stderr):
+        if log_stdout:
+            LOG.debug('Command stdout is: "%s"', stdout)
+        LOG.debug('Command stderr is: "%s"', stderr)
+
+    try:
+        result = processutils.execute(*cmd, **kwargs)
+    except FileNotFoundError:
+        with excutils.save_and_reraise_exception():
+            LOG.debug('Command not found: "%s"', ' '.join(map(str, cmd)))
+    except processutils.ProcessExecutionError as exc:
+        with excutils.save_and_reraise_exception():
+            _log(exc.stdout, exc.stderr)
+    else:
+        _log(result[0], result[1])
+        return result
+
+
+def try_execute(*cmd, **kwargs):
+    """The same as execute but returns None on error.
+
+    Executes and logs results from a system command. See docs for
+    oslo_concurrency.processutils.execute for usage.
+
+    Instead of raising an exception on failure, this method simply
+    returns None in case of failure.
+
+    :param cmd: positional arguments to pass to processutils.execute()
+    :param kwargs: keyword arguments to pass to processutils.execute()
+    :raises: UnknownArgumentError on receiving unknown arguments
+    :returns: tuple of (stdout, stderr) or None in some error cases
+    """
+    try:
+        return execute(*cmd, **kwargs)
+    except (processutils.ProcessExecutionError, OSError) as e:
+        LOG.debug('Command failed: %s', e)
 
 
 def mkfs(fs, path, label=None):
@@ -477,21 +506,16 @@ def wait_for_disk_to_become_available(device):
     :raises: IronicException If the disk fails to become
         available.
     """
-    retries = [0]
     pids = ['']
     stderr = ['']
     interval = CONF.disk_partitioner.check_device_interval
     max_retries = CONF.disk_partitioner.check_device_max_retries
 
-    def _wait_for_disk(device, retries, max_retries, pids, stderr):
+    def _wait_for_disk():
         # A regex is likely overkill here, but variations in fuser
         # means we should likely use it.
         fuser_pids_re = re.compile(r'\d+')
 
-        retries[0] += 1
-        if retries[0] > max_retries:
-            raise loopingcall.LoopingCallDone()
-
         # There are 'psmisc' and 'busybox' versions of the 'fuser' program. The
         # 'fuser' programs differ in how they output data to stderr.  The
         # busybox version does not output the filename to stderr, while the
@@ -516,7 +540,7 @@ def wait_for_disk_to_become_available(device):
                                run_as_root=True)
 
             if not out and not err:
-                raise loopingcall.LoopingCallDone()
+                return True
 
             stderr[0] = err
             # NOTE: findall() returns a list of matches, or an empty list if no
@@ -526,13 +550,16 @@ def wait_for_disk_to_become_available(device):
         except processutils.ProcessExecutionError as exc:
             LOG.warning('Failed to check the device %(device)s with fuser:'
                         ' %(err)s', {'device': device, 'err': exc})
+        return False
 
-    timer = loopingcall.FixedIntervalLoopingCall(
-        _wait_for_disk,
-        device, retries, max_retries, pids, stderr)
-    timer.start(interval=interval).wait()
-
-    if retries[0] > max_retries:
+    retry = tenacity.retry(
+        retry=tenacity.retry_if_result(lambda r: not r),
+        stop=tenacity.stop_after_attempt(max_retries),
+        wait=tenacity.wait_fixed(interval),
+        reraise=True)
+    try:
+        retry(_wait_for_disk)()
+    except tenacity.RetryError:
         if pids[0]:
             raise exception.IronicException(
                 _('Processes with the following PIDs are holding '
